@@ -5,7 +5,7 @@ import os
 import socket
 import threading
 
-SERVER_ADDRESS = '/tmp/py3status_uds'
+SERVER_ADDRESS = "/tmp/py3status_uds"
 MAX_SIZE = 1024
 
 REFRESH_EPILOG = """
@@ -47,34 +47,33 @@ examples:
         py3-cmd click --button 4 --index minutes timer  # up
         py3-cmd click --button 4 --index seconds timer  # up
 
-    width, height, relative_x, relative_y, x, y:
+    height, modifiers, relative_x, relative_y, width, x, y:
         # py3-cmd allows users to specify click events with
         # more options. however, there are no modules that
         # uses the aforementioned options.
 """
 # EXEC_EPILOG = ''
 INFORMATION = [
-    ('V', 'version', 'show version number and exit'),
-    ('v', 'verbose', 'enable verbose mode'),
+    ("V", "version", "show version number and exit"),
+    ("v", "verbose", "enable verbose mode"),
 ]
 SUBPARSERS = [
-    ('click', 'click modules', '+'),
-    ('refresh', 'refresh modules', '*'),
+    ("click", "click modules", "+"),
+    ("refresh", "refresh modules", "*"),
     # ('exec', 'execute methods', '+'),
 ]
 CLICK_OPTIONS = [
-    ('button', 'specify a button number (default %(default)s)'),
-    ('height', 'specify a height of the block, in pixel'),
-    ('index', 'specify an index value often found in modules'),
-    ('relative_x', 'specify relative X on the block, from the top left'),
-    ('relative_y', 'specify relative Y on the block, from the top left'),
-    ('width', 'specify a width of the block, in pixel'),
-    ('x', 'specify absolute X on the bar, from the top left'),
-    ('y', 'specify absolute Y on the bar, from the top left'),
-]
-REFRESH_OPTIONS = [
-    ('all', 'refresh all modules')
+    ("button", "specify a button number (default %(default)s)"),
+    ("height", "specify a height of the block, in pixel"),
+    ("index", "specify an index value often found in modules"),
+    ("modifiers", "specify modifiers, optionally joined by plus signs"),
+    ("relative_x", "specify relative X on the block, from the top left"),
+    ("relative_y", "specify relative Y on the block, from the top left"),
+    ("width", "specify a width of the block, in pixel"),
+    ("x", "specify absolute X on the bar, from the top left"),
+    ("y", "specify absolute Y on the bar, from the top left"),
 ]
+REFRESH_OPTIONS = [("all", "refresh all modules")]
 
 
 class CommandRunner:
@@ -83,7 +82,7 @@ class CommandRunner:
     """
 
     def __init__(self, py3_wrapper):
-        self.debug = py3_wrapper.config['debug']
+        self.debug = py3_wrapper.config["debug"]
         self.py3_wrapper = py3_wrapper
 
     def find_modules(self, requested_names):
@@ -92,37 +91,37 @@ class CommandRunner:
         """
         found_modules = set()
         for requested_name in requested_names:
-            is_instance = ' ' in requested_name
+            is_instance = " " in requested_name
 
             for module_name, module in self.py3_wrapper.output_modules.items():
-                if module['type'] == 'py3status':
-                    name = module['module'].module_nice_name
+                if module["type"] == "py3status":
+                    name = module["module"].module_nice_name
                 else:
-                    name = module['module'].module_name
+                    name = module["module"].module_name
                 if is_instance:
                     if requested_name == name:
                         found_modules.add(module_name)
                 else:
-                    if requested_name == name.split(' ')[0]:
+                    if requested_name == name.split(" ")[0]:
                         found_modules.add(module_name)
 
         if self.debug:
-            self.py3_wrapper.log('found %s' % found_modules)
+            self.py3_wrapper.log("found %s" % found_modules)
         return found_modules
 
     def refresh(self, data):
         """
         refresh the module(s)
         """
-        modules = data.get('module')
+        modules = data.get("module")
         # for i3status modules we have to refresh the whole i3status output.
         update_i3status = False
         for module_name in self.find_modules(modules):
             module = self.py3_wrapper.output_modules[module_name]
             if self.debug:
-                self.py3_wrapper.log('refresh %s' % module)
-            if module['type'] == 'py3status':
-                module['module'].force_update()
+                self.py3_wrapper.log("refresh %s" % module)
+            if module["type"] == "py3status":
+                module["module"].force_update()
             else:
                 update_i3status = True
         if update_i3status:
@@ -132,17 +131,17 @@ class CommandRunner:
         """
         send a click event to the module(s)
         """
-        modules = data.get('module')
+        modules = data.get("module")
         for module_name in self.find_modules(modules):
             module = self.py3_wrapper.output_modules[module_name]
-            if module['type'] == 'py3status':
-                name = module['module'].module_name
-                instance = module['module'].module_inst
+            if module["type"] == "py3status":
+                name = module["module"].module_name
+                instance = module["module"].module_inst
             else:
-                name = module['module'].name
-                instance = module['module'].instance
+                name = module["module"].name
+                instance = module["module"].instance
             # make an event
-            event = {'name': name, 'instance': instance}
+            event = {"name": name, "instance": instance}
             for name, message in CLICK_OPTIONS:
                 event[name] = data.get(name)
 
@@ -155,14 +154,14 @@ class CommandRunner:
         """
         check the given command and send to the correct dispatcher
         """
-        command = data.get('command')
+        command = data.get("command")
         if self.debug:
-            self.py3_wrapper.log('Running remote command %s' % command)
-        if command == 'refresh':
+            self.py3_wrapper.log("Running remote command %s" % command)
+        if command == "refresh":
             self.refresh(data)
-        elif command == 'refresh_all':
+        elif command == "refresh_all":
             self.py3_wrapper.refresh_modules()
-        elif command == 'click':
+        elif command == "click":
             self.click(data)
 
 
@@ -175,11 +174,11 @@ class CommandServer(threading.Thread):
     def __init__(self, py3_wrapper):
         threading.Thread.__init__(self)
 
-        self.debug = py3_wrapper.config['debug']
+        self.debug = py3_wrapper.config["debug"]
         self.py3_wrapper = py3_wrapper
 
         self.command_runner = CommandRunner(py3_wrapper)
-        server_address = '{}.{}'.format(SERVER_ADDRESS, os.getpid())
+        server_address = "{}.{}".format(SERVER_ADDRESS, os.getpid())
         self.server_address = server_address
 
         # Make sure the socket does not already exist
@@ -194,7 +193,7 @@ class CommandServer(threading.Thread):
         sock.bind(server_address)
 
         if self.debug:
-            self.py3_wrapper.log('Unix domain socket at %s' % server_address)
+            self.py3_wrapper.log("Unix domain socket at %s" % server_address)
 
         # Listen for incoming connections
         sock.listen(1)
@@ -220,37 +219,38 @@ class CommandServer(threading.Thread):
                 data = None
                 # Wait for a connection
                 if self.debug:
-                    self.py3_wrapper.log('waiting for a connection')
+                    self.py3_wrapper.log("waiting for a connection")
 
                 connection, client_address = self.sock.accept()
                 try:
                     if self.debug:
-                        self.py3_wrapper.log('connection from')
+                        self.py3_wrapper.log("connection from")
 
                     data = connection.recv(MAX_SIZE)
                     if data:
-                        data = json.loads(data.decode('utf-8'))
+                        data = json.loads(data.decode("utf-8"))
                         if self.debug:
-                            self.py3_wrapper.log(u'received %s' % data)
+                            self.py3_wrapper.log(u"received %s" % data)
                         self.command_runner.run_command(data)
                 finally:
                     # Clean up the connection
                     connection.close()
             except Exception:
                 if data:
-                    self.py3_wrapper.log('Command error')
+                    self.py3_wrapper.log("Command error")
                     self.py3_wrapper.log(data)
-                self.py3_wrapper.report_exception('command failed')
+                self.py3_wrapper.report_exception("command failed")
 
 
 def command_parser():
     """
     build and return our command parser
     """
+
     class Parser(argparse.ArgumentParser):
         # print usages and exit on errors
         def error(self, message):
-            print('\x1b[1;31merror: \x1b[0m{}'.format(message))
+            print("\x1b[1;31merror: \x1b[0m{}".format(message))
             self.print_help()
             self.exit(1)
 
@@ -266,62 +266,72 @@ def command_parser():
 
     # parser: add verbose, version
     for short, name, msg in INFORMATION:
-        short = '-%s' % short
-        arg = '--%s' % name
-        parser.add_argument(short, arg, action='store_true', help=msg)
+        short = "-%s" % short
+        arg = "--%s" % name
+        parser.add_argument(short, arg, action="store_true", help=msg)
 
     # make subparsers // ALIAS_DEPRECATION: remove metavar later
-    subparsers = parser.add_subparsers(dest='command', metavar='{click,refresh}')
+    subparsers = parser.add_subparsers(dest="command", metavar="{click,refresh}")
     sps = {}
 
     # subparsers: add refresh, click
     for name, msg, nargs in SUBPARSERS:
         sps[name] = subparsers.add_parser(
             name,
-            epilog=eval('{}_EPILOG'.format(name.upper())),
+            epilog=eval("{}_EPILOG".format(name.upper())),
             formatter_class=argparse.RawTextHelpFormatter,
             add_help=False,
-            help=msg
+            help=msg,
         )
-        sps[name].add_argument(nargs=nargs, dest='module', help='module name')
+        sps[name].add_argument(nargs=nargs, dest="module", help="module name")
 
     # ALIAS_DEPRECATION: subparsers: add click (aliases)
     buttons = {
-        'leftclick': 1, 'middleclick': 2, 'rightclick': 3,
-        'scrollup': 4, 'scrolldown': 5
+        "leftclick": 1,
+        "middleclick": 2,
+        "rightclick": 3,
+        "scrollup": 4,
+        "scrolldown": 5,
     }
     for name in sorted(buttons):
         sps[name] = subparsers.add_parser(name)
-        sps[name].add_argument(nargs='+', dest='module', help='module name')
+        sps[name].add_argument(nargs="+", dest="module", help="module name")
 
     # click subparser: add button, index, width, height, relative_{x,y}, x, y
-    sp = sps['click']
+    sp = sps["click"]
     for name, msg in CLICK_OPTIONS:
-        arg = '--{}'.format(name)
-        if name == 'button':
-            sp.add_argument(arg, metavar='INT', type=int, help=msg, default=1)
-        elif name == 'index':
-            sp.add_argument(arg, metavar='INDEX', help=msg)
+        arg = "--{}".format(name)
+        if name == "button":
+            sp.add_argument(arg, metavar="INT", type=int, help=msg, default=1)
+        elif name == "index":
+            sp.add_argument(arg, metavar="INDEX", help=msg)
+        elif name == "modifiers":
+            sp.add_argument(arg, metavar="KEY", help=msg)
         else:
-            sp.add_argument(arg, metavar='INT', type=int, help=msg)
+            sp.add_argument(arg, metavar="INT", type=int, help=msg)
 
     # refresh subparser: add all
-    sp = sps['refresh']
+    sp = sps["refresh"]
     for name, msg in REFRESH_OPTIONS:
-        arg = '--{}'.format(name)
-        sp.add_argument(arg, action='store_true', help=msg)
+        arg = "--{}".format(name)
+        sp.add_argument(arg, action="store_true", help=msg)
 
     # parse args, post-processing
     options = parser.parse_args()
 
-    if options.command == 'click':
+    if options.command == "click":
         # cast string index to int
         if options.index:
             try:
                 options.index = int(options.index)
             except ValueError:
                 pass
-    elif options.command == 'refresh':
+        # specify modifiers, optionally joined by plus signs
+        if options.modifiers:
+            options.modifiers = options.modifiers.split("+")
+        else:
+            options.modifiers = []
+    elif options.command == "refresh":
         # refresh all
         # ALL_DEPRECATION
         if options.module is None:
@@ -329,23 +339,24 @@ def command_parser():
         # end
         valid = False
         if options.all:  # keep this
-            options.command = 'refresh_all'
+            options.command = "refresh_all"
             options.module = []
             valid = True
-        if 'all' in options.module:  # remove this later
-            options.command = 'refresh_all'
+        if "all" in options.module:  # remove this later
+            options.command = "refresh_all"
             options.module = []
             valid = True
         if not options.module and not valid:
-            sps['refresh'].error('missing positional or optional arguments')
+            sps["refresh"].error("missing positional or optional arguments")
     elif options.version:
         # print version
         from platform import python_version
         from py3status.version import version
-        print('py3status {} (python {})'.format(version, python_version()))
+
+        print("py3status {} (python {})".format(version, python_version()))
         parser.exit()
     elif not options.command:
-        parser.error('too few arguments')
+        parser.error("too few arguments")
 
     # ALIAS_DEPRECATION
     alias = options.command in buttons
@@ -364,10 +375,10 @@ def command_parser():
     # ALIAS_DEPRECATION: Convert (click) aliases to buttons
     if alias:
         options.button = buttons[options.command]
-        options.command = 'click'
+        options.command = "click"
 
-    if options.command == 'click' and not new_modules:
-        sps[options.command].error('too few arguments')
+    if options.command == "click" and not new_modules:
+        sps[options.command].error("too few arguments")
     options.module = new_modules
 
     return options
@@ -390,12 +401,12 @@ def send_command():
             print(msg)
 
     options = command_parser()
-    msg = json.dumps(vars(options)).encode('utf-8')
+    msg = json.dumps(vars(options)).encode("utf-8")
     if len(msg) > MAX_SIZE:
-        verbose('Message length too long, max length (%s)' % MAX_SIZE)
+        verbose("Message length too long, max length (%s)" % MAX_SIZE)
 
     # find all likely socket addresses
-    uds_list = glob.glob('{}.[0-9]*'.format(SERVER_ADDRESS))
+    uds_list = glob.glob("{}.[0-9]*".format(SERVER_ADDRESS))
 
     verbose('message "%s"' % msg)
     for uds in uds_list:
@@ -403,12 +414,12 @@ def send_command():
         sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
 
         # Connect the socket to the port where the server is listening
-        verbose('connecting to %s' % uds)
+        verbose("connecting to %s" % uds)
         try:
             sock.connect(uds)
         except socket.error:
             # this is a stale socket so delete it
-            verbose('stale socket deleting')
+            verbose("stale socket deleting")
             try:
                 os.unlink(uds)
             except OSError:
@@ -416,8 +427,8 @@ def send_command():
             continue
         try:
             # Send data
-            verbose('sending')
+            verbose("sending")
             sock.sendall(msg)
         finally:
-            verbose('closing socket')
+            verbose("closing socket")
             sock.close()
