@@ -5,14 +5,19 @@ Display status of Dropbox daemon.
 Configuration parameters:
     cache_timeout: refresh interval for this module (default 10)
     format: display format for this module (default "Dropbox: {status}")
-    status_busy: text for placeholder {status} when Dropbox is busy (default None)
-    status_off: text for placeholder {status} when Dropbox isn't running (default "isn't running")
-    status_on: text for placeholder {status} when Dropbox is up to date (default "Up to date")
+    status_busy: text for placeholder {status} when Dropbox is busy
+        (default None)
+    status_off: text for placeholder {status} when Dropbox isn't running
+        (default "isn't running")
+    status_on: text for placeholder {status} when Dropbox is up to date
+        (default "Up to date")
 
 Value for `status_off` if not set:
     - Dropbox isn't running!
+
 Value for `status_on` if not set:
     - Up to date
+
 Values for `status_busy` if not set:
     - Connecting...
     - Starting...
@@ -30,6 +35,12 @@ Color options:
 Requires:
     dropbox-cli: command line interface for dropbox
 
+Notes:
+    Some distributions offer an option to install dropbox-cli. If you don't see
+    one for your distribution, then you need to download CLI Python script,
+    https://www.dropbox.com/help/desktop-web/linux-commands#commands, rename
+    it to `dropbox-cli`, make the script executable and available in your PATH.
+
 @author Tjaart van der Walt (github:tjaartvdwalt)
 @license BSD
 
@@ -43,12 +54,13 @@ off
 {'color': '#FF0000', 'full_text': "Dropbox: isn't running"}
 """
 
-STRING_NOT_INSTALLED = 'not installed'
+STRING_NOT_INSTALLED = "not installed"
 
 
 class Py3status:
     """
     """
+
     # available configuration parameters
     cache_timeout = 10
     format = "Dropbox: {status}"
@@ -58,21 +70,23 @@ class Py3status:
 
     class Meta:
         deprecated = {
-            'format_fix_unnamed_param': [
+            "format_fix_unnamed_param": [
                 {
-                    'param': 'format',
-                    'placeholder': 'status',
-                    'msg': '{} should not be used in format use `{status}`',
-                },
-            ],
+                    "param": "format",
+                    "placeholder": "status",
+                    "msg": "{} should not be used in format use `{status}`",
+                }
+            ]
         }
 
     def post_config_hook(self):
-        if not self.py3.check_commands('dropbox-cli'):
+        if not self.py3.check_commands("dropbox-cli"):
             raise Exception(STRING_NOT_INSTALLED)
 
     def dropbox(self):
-        status = self.py3.command_output('dropbox-cli status').splitlines()[0]
+        status = self.py3.command_output(
+            "dropbox-cli status", localized=True
+        ).splitlines()[0]
 
         if status == "Dropbox isn't running!":
             color = self.py3.COLOR_BAD
@@ -86,9 +100,9 @@ class Py3status:
                 status = self.status_busy
 
         return {
-            'cached_until': self.py3.time_in(self.cache_timeout),
-            'color': color,
-            'full_text': self.py3.safe_format(self.format, {'status': status})
+            "cached_until": self.py3.time_in(self.cache_timeout),
+            "color": color,
+            "full_text": self.py3.safe_format(self.format, {"status": status}),
         }
 
 
@@ -97,4 +111,5 @@ if __name__ == "__main__":
     Run module in test mode.
     """
     from py3status.module_test import module_test
+
     module_test(Py3status)
